unit thy_opts;

interface

uses
  Forms, StdCtrls, ComCtrls, Controls, Buttons, Classes, LanManager;

type
  pBadUser = ^tBadUser;
  tBadUser = record
    uName    : string;
    uReply   : string;
    uKeywords: string;
    uTimeFrom: string;
    uTimeTo  : string;
  end;

  TfSetup = class(TForm)
    RB1: TRadioButton;
    RB2: TRadioButton;
    GroupBox1: TGroupBox;
    TB1: TTrackBar;
    Memo1: TMemo;
    Label2: TLabel;
    Button1: TButton;
    Button2: TButton;
    ListBox1: TListBox;
    Label3: TLabel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    Button3: TButton;
    SCCB: TCheckBox;
    SYMCB: TCheckBox;
    BUList: TListBox;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    LS1: TLanStorage;
    Label1: TLabel;
    CB1: TComboBox;
    procedure RB2Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TB1Change(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure BUListClick(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure CB1Change(Sender: TObject);
    procedure LS1CanChangeComponent(Sender: TObject;
      aComponent: TComponent; var CanChange: Boolean);
  private
    { Private declarations }
    fBadUsers: tStrings;
    function GetAutoReply: Boolean;
    function GetReplyDelay: Integer;
    procedure SetAutoReply(aValue: Boolean);
    procedure SetReplyDelay(aValue: Integer);
    function  BUList2Str: string;
    procedure Str2BUList(aStr: string);
    procedure FillBU(aBU: pBadUser);
    function  TimeGood(aBU: pBadUser): Boolean;
  public
    { Public declarations }
    procedure DoSetLanID;
    function IsBadUser(const aUserName, aMessage: string): string;
    property AutoReply : Boolean read GetAutoReply write SetAutoReply;
    property ReplyDelay: Integer read GetReplyDelay write SetReplyDelay;
  end;

var
  fSetup: TfSetup;

implementation

uses
  Dialogs, SysUtils, thy_main, thy_keyw {, thy_rmnd}, thy_lanconsts;

{$R *.DFM}

procedure TfSetup.RB2Click(Sender: TObject);
begin
  GroupBox1.Visible := RB2.Checked;
{$IFDEF CNT}
  if isNT and RB2.Checked then ShowMessage(lS1[sWARWNW]);
{$ENDIF}  
end;

function tfSetup.GetAutoReply: Boolean;
begin
  Result := RB2.Checked;
end;

function tfSetup.GetReplyDelay: Integer;
begin
  Result := TB1.Position;
end;

procedure tfSetup.SetAutoReply(aValue: Boolean);
begin
  if (RB2.Checked <> aValue) then begin
    RB2.Checked := aValue;
    //RB2Click(Self);
  end;
end;

procedure tfSetup.SetReplyDelay(aValue: Integer);
begin
  if (TB1.Position <> aValue) then begin
    TB1.Position := aValue;
  end;
end;

const
  divF = #6#6#6;
  divU = #7#7#7;

function TfSetup.BUList2Str: string;
var
  i: Integer;
begin
  Result := '';
  for i := 0 to fBadUsers.Count - 1 do
    with pBadUser(fBadUsers.Objects[i])^ do
      Result := Result + uName + divF + uKeywords + divF + uReply + divF + uTimeFrom + divF + uTimeTo + divU;
end;

function lU2Str(lU: pBadUser): string;
begin
  Result := lU.uName + ' => ';
  if (lU.uKeywords <> '') then Result := Result + lU.uKeywords
			  else Result := Result + Format(fSetup.lS1[sFT], [lU.uTimeFrom, lU.uTimeTo]);
end;

procedure TfSetup.Str2BUList(aStr: string);
var
  lU: pBadUser;
  lP: Integer;
  lS: string;

  function NextSS: string;
  var
    lP: Integer;
  begin
    lP := Pos(divF, lS);
    if (lP > 0) then begin
      Result := Copy(lS, 1, lP - 1);
      Delete(lS, 1, lP + Length(divF) - 1);
    end
    else begin
      Result := lS;
      lS := '';
    end;
  end;

begin
  fBadUsers.Clear;
  lP := Pos(divU, aStr);
  while (lP > 0) do begin
    lS := Copy(aStr, 1, lP - 1);
    Delete(aStr, 1, lP + Length(divU) - 1);
    New(lU);
    lU.uName     := NextSS;
    lU.uKeywords := NextSS;
    lU.uReply    := NextSS;
    lU.uTimeFrom := NextSS;
    lU.uTimeTo   := NextSS;
    fBadUsers.AddObject(IntToStr(fBadUsers.Count), Pointer(lU));
    BUList.Items.Add(lU2Str(lU));
    lP := Pos(divU, aStr);
  end;
end;

procedure TfSetup.Button2Click(Sender: TObject);
begin
  with fMain do begin
    if AutoReply then fMain.WriteRegValue(sAutoReplyEnabled, 'Y')
		 else fMain.WriteRegValue(sAutoReplyEnabled, 'no');
    if AutoReply then begin
      WriteRegValue(sAutoReplyDelay, IntToStr(ReplyDelay));
      WriteRegValue(sAutoReplyMessage, Memo1.Text);
      WriteRegValue(sUserFilter, ListBox1.Items.Text);
    end;
    if SCCB.Checked then WriteRegValue(sStartScan, '1')
		    else WriteRegValue(sStartScan, '0');
    if SYMCB.Checked then WriteRegValue(sSaveYM, '1')
		     else WriteRegValue(sSaveYM, '0');
    WriteRegValue(sBUList, BUList2Str);
    WriteRegValue(sCBLan, IntToStr(CB1.ItemIndex));
  end;
end;

procedure TfSetup.SpeedButton1Click(Sender: TObject);
var
  lS: string;
begin
  lS := 'PowerChute';
  if InputQuery(lS1[sENTSTR], lS1[sENTPO], lS) then ListBox1.Items.Add(lS);
end;

procedure TfSetup.SpeedButton2Click(Sender: TObject);
begin
  if (ListBox1.ItemIndex >= 0) then ListBox1.Items.Delete(ListBox1.ItemIndex);
end;

procedure TfSetup.FormCreate(Sender: TObject);
var
  lZ: Integer;
  lC: Integer;
  lU: string;
   i: Integer;
begin
  if not Terminated then begin
    with CB1.Items do begin
      Clear;
      for i := 0 to fMain.lM1.LanCount - 1 do Add(fMain.lM1.LanName[i]);
    end;
    fBadUsers := tStringList.Create;
    BUListClick(Self);
    lU := fMain.ReadRegValue(sAutoReplyEnabled);
    if (lU <> '') then AutoReply := (lU[1] in ['Y', 'y']);
    if AutoReply then begin
      lU := fMain.ReadRegValue(sAutoReplyDelay);
      lC := -1;
      if (lU <> '') then begin
	Val(lU, lZ, lC);
	if (lC = 0) then ReplyDelay := lZ;
      end;
      if (lC = 0) then begin
	Memo1.Text := fMain.ReadRegValue(sAutoReplyMessage);
	ListBox1.Items.Text := fMain.ReadRegValue(sUserFilter);
      end;
    end;
    SCCB.Checked := (fMain.ReadRegValue(sStartScan) = '1');
    SYMCB.Checked := (fMain.ReadRegValue(sSaveYM) = '1');
    Str2BUList(fMain.ReadRegValue(sBUList));
    CB1.ItemIndex := StrToIntDef(fMain.ReadRegValue(sCBLan), 0);
    DoSetLanID;
  end;
end;

procedure TfSetup.TB1Change(Sender: TObject);
begin
  Hint := ' = ' + IntToStr(TB1.Position) + ' sec.';
end;

procedure TfSetup.Button3Click(Sender: TObject);
begin
  //fRemind.ShowModal;
  //ShowMessage('Under construction.');
end;

procedure TfSetup.FillBU(aBU: pBadUser);
begin
  FillChar(aBU^, SizeOf(aBU^), #0);	// to clear uKeywords & uTimexxx
  aBU.uName     := fKeyw.Edit1.Text;
  aBU.uReply    := fKeyw.Memo1.Text;
  if (fKeyw.RBG1.ItemIndex = 0) then aBU.uKeywords := fKeyw.Edit2.Text
  else begin
    aBU.uTimeFrom := TimeToStr(fKeyw.DTPF.Time);
    aBU.uTimeTo   := TimeToStr(fKeyw.DTPT.Time);
  end;
end;

procedure TfSetup.Button4Click(Sender: TObject);
var
  lU: pBadUser;
begin
  // add Bad User
  if fKeyw.AddBU then begin
    New(lU);
    FillBU(lU);
    fBadUsers.AddObject(IntToStr(fBadUsers.Count), Pointer(lU));
    BUList.Items.Add(lU2Str(lU));
    BUListClick(Self);
  end;
end;

procedure TfSetup.FormDestroy(Sender: TObject);
begin
  fBadUsers.Free;
end;

procedure TfSetup.Button6Click(Sender: TObject);
var
  lU: pBadUser;
   i: Integer;
begin
  // EDIT Bad User
  i := BUList.ItemIndex;
  if (i >= 0) then begin
    Pointer(lU) := fBadUsers.Objects[i];
    with fKeyw do begin
      Edit1.Text := lU.uName;
      Memo1.Text := lU.uReply;
      RBG1.ItemIndex := 0;
      if (lU.uKeywords <> '') then Edit2.Text := lU.uKeywords
      else begin
	DTPF.Time := StrToTime(lU.uTimeFrom);
	DTPT.Time := StrToTime(lU.uTimeTo);
	RBG1.ItemIndex := 1;
      end;
      if EditBU then begin
	FillBU(lU);
	BUList.Items[i] := lU2Str(lU);
      end;
    end;
  end;
end;

procedure TfSetup.BUListClick(Sender: TObject);
begin
  Button6.Enabled := (BUList.ItemIndex >= 0);
  Button5.Enabled := Button6.Enabled;
end;

procedure TfSetup.Button5Click(Sender: TObject);
var
  i: Integer;
begin
  if Sure(lS1[sDTI]) then begin
    i := BUList.ItemIndex;
    if (i >= 0) then begin
      Dispose(pBadUser(fBadUsers.Objects[i]));
      fBadUsers.Delete(i);
      BUList.Items.Delete(i);
      BUListClick(Self);
    end
    else Button5.Enabled := False;
  end;  
end;

function AllKeywords(const aMsg, aKeywords: string): Boolean;
var
  lM: string;
  lW: string;
  lP: Integer;

  function NextWord: string;
  var
    InWord: Boolean;
  begin
    Result := '';
    InWord := False;
    while (lP <= Length(aKeywords)) do begin
      if (Ord(aKeywords[lP]) < 32) then
	if InWord then Break
		  else Inc(lP)
      else begin
	Result := Result + aKeywords[lP];
	Inc(lP);
	InWord := True;
      end;
    end;
    Result := AnsiUpperCase(Result);
  end;

begin
  lM := AnsiUpperCase(aMsg);
  lP := 1;
  Result := True;
  repeat
    lW := NextWord;
    if (lW <> '') then
      if (Pos(lW, lM) < 1) then begin
	Result := False;
	Break;
      end;
  until (lW = '');
end;

function TfSetup.TimeGood(aBU: pBadUser): Boolean;
begin
  if (aBU.uTimeFrom <> '') and (aBU.uTimeTo <> '') then Result := (StrToTime(aBU.uTimeFrom) <= Time) and (Time <= StrToTime(aBU.uTimeTo))
                                                   else Result := False;
end;

function TfSetup.IsBadUser(const aUserName, aMessage: string): string;
var
   i : Integer;
  lU : pBadUser;
  lOK: Boolean;
begin
  Result := '';
  for i := 0 to fBadUsers.Count - 1 do begin
    lU := pBadUser(fBadUsers.Objects[i]);
    if (UpperCase(lU.uName) = UpperCase(aUserName)) then begin
      if (lU.uKeywords <> '') then lOK := AllKeywords(aMessage, lU.uKeywords)
			      else lOK := TimeGood(lU);
      if lOK then begin
	Result := lU.uReply;
	Break;
      end;
    end;  
  end;
end;

procedure TfSetup.DoSetLanID;
begin
  if (CB1.ItemIndex >= 0) then fMain.LM1.LanID := CB1.ItemIndex;
end;

procedure TfSetup.CB1Change(Sender: TObject);
begin
  DoSetLanID;
end;

procedure TfSetup.LS1CanChangeComponent(Sender: TObject;
  aComponent: TComponent; var CanChange: Boolean);
begin
  CanChange := (aComponent <> CB1); 
end;

end.

