
unit thy_talk;

interface

uses
  Windows, Buttons, Controls, ExtCtrls, ComCtrls, StdCtrls, Classes, Mover, Forms,
  LanManager;

type
  TfChat = class(TForm)
    Memo1: TMemo;
    Memo2: TMemo;
    Mover1: TMover;
    SB: TStatusBar;
    Panel1: TPanel;
    StopBtn: TSpeedButton;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    LS1: TLanStorage;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Memo2Change(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure LS1CanChangeComponent(Sender: TObject;
      aComponent: TComponent; var CanChange: Boolean);
  private
    { Private declarations }
    procedure SendMemo;
  public
    { Public declarations }
    CompName: string;
    SideDown: Boolean;
    UserName: string;
  end;

var
  fChat: TfChat;

procedure StartTalk(const WithComp, WithUser: string);
procedure StopTalk(const WithComp: string);
procedure WriteReply(const WithComp, WithUser, Reply: string);
procedure ClearTalks;

var
  lTalks: tStrings;

implementation

uses thy_main, thy_msgs, thy_opts, thy_favor, Dialogs, thy_lanconsts;

{$R *.DFM}

const
  sJustFreeMe = 'Please, set me free!';

procedure ClearTalks;
var
  lI: Integer;
begin
  lI := 0;
  while (lI < lTalks.Count) do begin
    if (lTalks[lI] = sJustFreeMe) then begin
      tfChat(lTalks.Objects[lI]).Free;
      lTalks.Delete(lI);
      Dec(lI);
    end;
    Inc(lI);
  end;
end;

procedure StartTalk(const WithComp, WithUser: string);
var
  lF: tfChat;
  lI: Integer;
begin
  ClearTalks;
  lI := lTalks.IndexOf(WithComp);
  if (lI >= 0) then lF := tfChat(lTalks.Objects[lI])
  else begin
    lF := tfChat.Create(nil);
    lTalks.AddObject(WithComp, lF);
    lF.Memo2.Clear;
    lF.Memo1.Clear;
  end;
  with lF do begin
    CompName := WithComp;
    Username := WithUser;
    Caption  := lS1[sCHW] + WithUser+ '@' + fMain.Alias(WithComp);
    SB.SimpleText := lS1[sCTNG];
    Show;
    Application.BringToFront;
    SideDown := False;
  end;
end;

procedure WriteReply(const WithComp, WithUser, Reply: string);
var
  lI: Integer;
begin
  lI := lTalks.IndexOf(WithComp);
  if (lI >= 0) then tfChat(lTalks.Objects[lI]).Memo1.Text := Reply
  else begin
    fMessages.AddMsg(WithComp, WithUser, Reply);
    if fSetup.AutoReply and
       not (Pos(fChat.LS1[sStdSign], Reply) > 0) {do not auto reply on AutoReply!} then fMain.AddWaiter(WithComp);
  end;
end;

procedure StopTalk(const WithComp: string);
var
  lI: Integer;
  lF: tfChat;
begin
  lI := lTalks.IndexOf(WithComp);
  if (lI >= 0) then begin
    lF := tfChat(lTalks.Objects[lI]);
    lF.SideDown := True;
    lF.Close;
  end;
end;

procedure TfChat.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := True;
  ClearTalks;
  if (lTalks.Count > 0) then
    if SideDown then fMessages.AddMsg(fMain.CompName, 'PopChat', (fMain.Alias(CompName) + lS1[sST]))
    else begin
      CanClose := (MessageDlg(lS1[sSGT] + fMain.Alias(CompName) + '?', mtConfirmation, mbYesNoCancel, 0) = mrYes);
      if CanClose then _SendMessage(CompName, sTheEndComplete);
    end;
end;

procedure TfChat.FormClose(Sender: TObject; var Action: TCloseAction);
var
  lI: Integer;
begin
  lI := lTalks.IndexOf(CompName);
  if (lI >= 0) then lTalks[lI] := sJustFreeMe;
end;

procedure tfChat.SendMemo;
var
  lS: string;
begin
  lS := Memo2.Text;
  if (lS = '') then lS := ' ';
  _SendMessage(CompName, lS);
end;

procedure TfChat.Memo2Change(Sender: TObject);
begin
  if not SideDown then SendMemo;
  if (Length(Memo2.Text) > 300) then SB.SimpleText := lS1[sWMTB]
				else SB.SimpleText := lS1[sCTNG];
end;

procedure TfChat.StopBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TfChat.SpeedButton1Click(Sender: TObject);
begin
  Memo2.Clear;
end;

procedure TfChat.SpeedButton2Click(Sender: TObject);
begin
  fFavor.AddComp(CompName, '');
end;

procedure TfChat.FormCreate(Sender: TObject);
begin
  SideDown := True;
end;

procedure TfChat.LS1CanChangeComponent(Sender: TObject;
  aComponent: TComponent; var CanChange: Boolean);
begin
  CanChange := (aComponent <> Memo2);
end;

initialization
  lTalks  := tStringList.Create;
  lWaiters := tStringList.Create;

finalization
  ClearTalks;
  lTalks.Free;
  lWaiters.Free;

end.

