unit thy_umsg;

interface

uses Windows, StdCtrls, Buttons, Controls, Classes, Forms, Messages,
  LanManager;

type
  TDualListDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Memo1: TMemo;
    GroupBox1: TGroupBox;
    SrcLabel: TLabel;
    SrcList: TListBox;
    DstLabel: TLabel;
    ExAllBtn: TSpeedButton;
    ExcludeBtn: TSpeedButton;
    IncAllBtn: TSpeedButton;
    IncludeBtn: TSpeedButton;
    DstList: TListBox;
    SrcEdit: TEdit;
    SpeedButton1: TSpeedButton;
    Label1: TLabel;
    SpeedButton2: TSpeedButton;
    Label2: TLabel;
    DstDomains: TListBox;
    SBDInclude: TSpeedButton;
    SBDExclude: TSpeedButton;
    Label3: TLabel;
    Label4: TLabel;
    SrcDomains: TListBox;
    SpeedButton5: TSpeedButton;
    Label5: TLabel;
    SrcDEdit: TEdit;
    SpeedButton3: TSpeedButton;
    LS1: TLanStorage;
    procedure IncludeBtnClick(Sender: TObject);
    procedure ExcludeBtnClick(Sender: TObject);
    procedure IncAllBtnClick(Sender: TObject);
    procedure ExcAllBtnClick(Sender: TObject);
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure SetItem(List: TListBox; Index: Integer);
    function GetFirstSelection(List: TCustomListBox): Integer;
    procedure SetButtons;
    procedure CancelBtnClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SrcListDblClick(Sender: TObject);
    procedure DstListDblClick(Sender: TObject);
    procedure DstListClick(Sender: TObject);
    procedure SrcListClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure Memo1Change(Sender: TObject);
    procedure SpeedButton5Click(Sender: TObject);
    procedure SBDIncludeClick(Sender: TObject);
    procedure SBDExcludeClick(Sender: TObject);
    procedure SrcDomainsDblClick(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure DstDomainsDblClick(Sender: TObject);
  private
    { Private declarations }
    procedure MyOnKeyD(var Msg: tWMKeyDown); message WM_KEYDown;
    procedure Left2Right;
    procedure Right2Left;
    procedure Left2Right2;
    procedure Right2Left2;
  public
    { Public declarations }
  end;

var
  DualListDlg: TDualListDlg;

procedure SendMsg2Users(const aUser, aMsg: string);

implementation

uses
  SysUtils, thy_main, thy_msgs, thy_opts, thy_favor, thy_lanconsts;

{$R *.DFM}

procedure SendMsg2Users(const aUser, aMsg: string);
var
  i: Integer;
begin
  with DualListDlg do begin
    SrcList.Items.Clear;
    SrcList.Items.Assign(fMain.fUsers);
    SrcList.Items.Delete(SrcList.Items.IndexOf(aUser));
    //while (Pos(';', aUser) > 0) do aUser[Pos(';', aUser)] := #10;
    DstList.Items.Text := aUser;
    if (aMsg <> '') then begin
      Memo1.Text := aMsg;
      for i := 0 to Memo1.Lines.Count - 1 do Memo1.Lines[i] := '> ' + Memo1.Lines[i];
      Memo1.Lines.Insert(0, aUser + lS1[sWrote]);
      Memo1.Lines.Add('');
    end
    else Memo1.Text := '';
    Show;
  end;
end;

procedure TDualListDlg.Left2Right;
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcList);
  MoveSelected(SrcList, DstList.Items);
  SetItem(SrcList, Index);
end;

procedure TDualListDlg.IncludeBtnClick(Sender: TObject);
begin
  Left2Right;
end;

procedure TDualListDlg.Right2Left;
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstList);
  MoveSelected(DstList, SrcList.Items);
  SetItem(DstList, Index);
end;

procedure TDualListDlg.ExcludeBtnClick(Sender: TObject);
begin
  Right2Left;
end;

procedure TDualListDlg.IncAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to SrcList.Items.Count - 1 do
    DstList.Items.AddObject(SrcList.Items[I], SrcList.Items.Objects[I]);
  SrcList.Items.Clear;
  SetItem(SrcList, 0);
end;

procedure TDualListDlg.ExcAllBtnClick(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to DstList.Items.Count - 1 do
    SrcList.Items.AddObject(DstList.Items[I], DstList.Items.Objects[I]);
  DstList.Items.Clear;
  SetItem(DstList, 0);
end;

procedure TDualListDlg.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if (List.Selected[I]) then begin
      if (Items.IndexOf(List.Items[I]) = -1) then Items.AddObject(List.Items[I], List.Items.Objects[I]);
      List.Items.Delete(I);
    end;
end;

procedure TDualListDlg.SetButtons;
var
  SrcEmpty,
  DstEmpty: Boolean;
  SrcEmpty2,
  DstEmpty2: Boolean;
begin
  SrcEmpty := (SrcList.Items.Count < 1);
  DstEmpty := (DstList.Items.Count < 1);
  SrcEmpty2 := (SrcDomains.Items.Count < 1);
  DstEmpty2 := (DstDomains.Items.Count < 1);
  IncludeBtn.Enabled := not SrcEmpty;
  IncAllBtn.Enabled  := not SrcEmpty;
  ExcludeBtn.Enabled := not DstEmpty;
  ExAllBtn.Enabled   := not DstEmpty;
  SBDInclude.Enabled := not SrcEmpty2;
  SBDExclude.Enabled := not DstEmpty2;
  OkBtn.Enabled := (not DstEmpty or not DstEmpty2) and (Length(Memo1.Text) > 0);
end;

function TDualListDlg.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

procedure TDualListDlg.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else
      if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;

procedure TDualListDlg.CancelBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TDualListDlg.OKBtnClick(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to DstList.Items.Count - 1 do _SendMessage(DstList.Items[i], Memo1.Text);
  for i := 0 to DstDomains.Items.Count - 1 do _SendMessage(DstDomains.Items[i], Memo1.Text);
  if fSetup.SYMCB.Checked then fMessages.AddMsg(fMain.CompName, 'YOU', Memo1.Text);
  //ModalResult := mrOK;
  Close;
end;

procedure TDualListDlg.SpeedButton1Click(Sender: TObject);
var
  lS: string;
begin
  if (SrcEdit.Text <> '') then begin
    lS := AnsiUpperCase(SrcEdit.Text);
    if (DstList.Items.IndexOf(lS) < 0) then DstList.Items.Add(lS);
  end;
  SetButtons;
end;

procedure TDualListDlg.SrcListDblClick(Sender: TObject);
begin
  Left2Right;
end;

procedure TDualListDlg.DstListDblClick(Sender: TObject);
begin
  Right2Left;
end;

procedure TDualListDlg.DstListClick(Sender: TObject);
begin
  SetButtons;
end;

procedure TDualListDlg.SrcListClick(Sender: TObject);
begin
  SetButtons;
end;

procedure TDualListDlg.FormShow(Sender: TObject);
begin
  SrcDomains.Items.Assign(fMain.fDomains);
  Memo1.SetFocus;
  SetButtons;
end;

procedure TDualListDlg.SpeedButton2Click(Sender: TObject);
var
  I: Integer;
  S: string;
begin
  S := '';
  for I := 0 to DstList.Items.Count - 1 do
    if DstList.Selected[i] then S := S + DstList.Items[I] + ';';
  fFavor.AddComp(S, '');
end;

procedure TDualListDlg.Memo1Change(Sender: TObject);
begin
  SetButtons;
end;

procedure TDualListDlg.SpeedButton5Click(Sender: TObject);
var
  lS: string;
begin
  lS := Caption;
  Caption := lS1[sSLNPW];
  Application.ProcessMessages;
  fMain.RefreshUsers;
  Caption := lS;
  SrcList.Items.Assign(fMain.fUsers);
  SrcDomains.Items.Assign(fMain.fDomains);
end;

procedure tDualListDlg.Left2Right2;
var
  Index: Integer;
begin
  Index := GetFirstSelection(SrcDomains);
  MoveSelected(SrcDomains, DstDomains.Items);
  SetItem(SrcDomains, Index);
end;

procedure TDualListDlg.SBDIncludeClick(Sender: TObject);
begin
  Left2Right2;
end;

procedure tDualListDlg.Right2Left2;
var
  Index: Integer;
begin
  Index := GetFirstSelection(DstDomains);
  MoveSelected(DstDomains, SrcDomains.Items);
  SetItem(DstDomains, Index);
end;

procedure TDualListDlg.SBDExcludeClick(Sender: TObject);
begin
  Right2Left2;
end;

procedure TDualListDlg.SrcDomainsDblClick(Sender: TObject);
begin
  Left2Right2;
end;

procedure TDualListDlg.SpeedButton3Click(Sender: TObject);
var
  lS: string;
begin
  if (SrcDEdit.Text <> '') then begin
    lS := AnsiUpperCase(SrcDEdit.Text);
    if (DstDomains.Items.IndexOf(lS) < 0) then DstDomains.Items.Add(lS);
  end;
  SetButtons;
end;

procedure TDualListDlg.DstDomainsDblClick(Sender: TObject);
begin
  Right2Left2;
end;

procedure TDualListDlg.MyOnKeyD(var Msg: tWMKeyDown); 
begin
  if (Msg.CharCode = 21) then Close;
end;

end.

